@@ -15,6 +15,8 @@ MySQLError = _mysql.MySQLError
 OperationalError = _mysql.OperationalError
 DatabaseError = _mysql.DatabaseError
 
+encoders = dict([ (k, v) for k, v in conversions.items() if not isinstance(k, int) ])
+
 def connect(*args, **kwargs):
     return Connection(*args, **kwargs)
 
@@ -39,8 +41,6 @@ class Connection(object):
         self.database = database
         self.max_idle_time = max_idle_time
 
-        self.encoders = dict([ (k, v) for k, v in conversions.items() if not isinstance(k, int) ])
-
         sys_vars = {
             "character_set_server":  "utf8",
             "collation_server":      "utf8_general_ci"
@@ -111,19 +111,19 @@ class Connection(object):
             self.reconnect()
         self._last_use_time = time.time()
 
-    def debug_query(self, query, *parameters, **kwparamaters):
-        return self._query(query, parameters, kwparamaters, debug=True)
+    def debug_query(self, query, *parameters, **kwparameters):
+        return self._query(query, parameters, kwparameters, debug=True)
 
-    def query(self, query, *parameters, **kwparamaters):
+    def query(self, query, *parameters, **kwparameters):
         """
         Query the connection and return the rows (or affected rows if not a
         select query).  Mysql errors will be propogated as exceptions.
         """
-        return self._query(query, parameters, kwparamaters)
+        return self._query(query, parameters, kwparameters)
 
-    def get(self, query, *parameters, **kwparamaters):
+    def get(self, query, *parameters, **kwparameters):
         """Returns the first row returned for the given query."""
-        rows = self._query(query, parameters, kwparamaters)
+        rows = self._query(query, parameters, kwparameters)
         if not rows:
             return None
         elif not isinstance(rows, list):
@@ -135,18 +135,18 @@ class Connection(object):
 
     # rowcount is a more reasonable default return value than lastrowid,
     # but for historical compatibility execute() must return lastrowid.
-    def execute(self, query, *parameters, **kwparamaters):
+    def execute(self, query, *parameters, **kwparameters):
         """Executes the given query, returning the lastrowid from the query."""
-        return self.execute_lastrowid(query, *parameters, **kwparamaters)
+        return self.execute_lastrowid(query, *parameters, **kwparameters)
 
-    def execute_lastrowid(self, query, *parameters, **kwparamaters):
+    def execute_lastrowid(self, query, *parameters, **kwparameters):
         """Executes the given query, returning the lastrowid from the query."""
-        self._execute(query, parameters, kwparamaters)
+        self._execute(query, parameters, kwparameters)
         self._result = self._db.store_result()
         return self._db.insert_id()
 
-    def _query(self, query, parameters, kwparamaters, debug=False):
-        self._execute(query, parameters, kwparamaters, debug)
+    def _query(self, query, parameters, kwparameters, debug=False):
+        self._execute(query, parameters, kwparameters, debug)
         self._result = self._db.use_result()
         if self._result is None:
             return self._rowcount
@@ -155,31 +155,11 @@ class Connection(object):
         ret = SelectResult(fields, rows)
         return ret
 
-    def _escape_unicode(self, param):
-        if isinstance(param, unicode):
-            return param.encode(self._db.character_set_name())
-        else:
-            return param
-
-    def _escape(self, param):
-        if isinstance(param, (list, tuple)):
-            param = [self._escape_unicode(p) for p in param]
-            return ','.join(_mysql.escape_sequence(param, self.encoders))
-        else:
-            return self._db.escape(self._escape_unicode(param), self.encoders)
-
-    def _execute(self, query, parameters, kwparamaters, debug=False):
-        if parameters and kwparamaters:
+    def _execute(self, query, parameters, kwparameters, debug=False):
+        if parameters and kwparameters:
             raise ValueError('database.py querying functions can receive *args or **kwargs, but not both')
 
-        if parameters:
-            query = query % tuple([self._escape(item) for item in parameters])
-        elif kwparamaters:
-            query = query % dict((key, self._escape(item)) for key, item in kwparamaters.iteritems())
-
-        if isinstance(query, unicode):
-            query = query.encode(self._db.character_set_name())
-
+        query = escape_query(query, parameters or kwparameters)
         if debug:
             print query
 
@@ -209,3 +189,26 @@ class SelectResult(list):
         if isinstance(i, slice):
             return SelectResult(self.fieldnames, self.rows[i])
         return list.__getitem__(self, i)
+
+def escape_query(query, parameters):
+    if isinstance(parameters, (list, tuple)):
+        query = query % tuple(map(_escape, parameters))
+    elif isinstance(parameters, dict):
+        query = query % dict(map(lambda (key, item): (key, _escape(item)), parameters.iteritems()))
+    else:
+        assert False, 'not sure what to do with parameters of type %s' % type(parameters)
+
+    return _escape_unicode(query)
+
+def _escape_unicode(param):
+    if isinstance(param, unicode):
+        return param.encode('utf-8')
+    else:
+        return param
+
+def _escape(param):
+    if isinstance(param, (list, tuple)):
+        param = [_escape_unicode(p) for p in param]
+        return ','.join(_mysql.escape_sequence(param, encoders))
+    else:
+        return _mysql.escape(_escape_unicode(param), encoders)
